
#include "base64.h"

/* Convert to Base64 encoding */
static char to_base64[64] = {
	'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 
	'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 
	'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 
	'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', 
	'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/' 
	};
#define PAD	'='		/* Trailing pad character */


unsigned int base64_encode(const char *src, unsigned int srclen, char *dst, unsigned int dstlen)
{
	/*unsigned int needed = (srclen*4)/3+5;*/
	unsigned char sextet[4];
	unsigned int i, index=0, o=0;

/*
fprintf(stderr, "Encoding %d chars...", srclen);
*/
	for ( i=0; i<srclen; ++i ) {
		switch (index++) {
			case 0: sextet[0]  = ((src[i]&0xfc)>>2);
					sextet[1]  = ((src[i]&0x03)<<4);
					break;
			case 1: sextet[1] |= ((src[i]&0xf0)>>4);
					sextet[2]  = ((src[i]&0x0f)<<2);
					break;
			case 2: sextet[2] |= ((src[i]&0xc0)>>6);
					sextet[3]  = (src[i]&0x3f);
					/* Now output the encoded data */
					for ( index=0; index<4; ++index )
						dst[o++] = to_base64[sextet[index]];
					index=0;
					break;
			default:  /* Never reached */ ;
		}
	}
	if ( index ) { /* We must flush the output */
		for ( i=0; i<=index; ++i )
			dst[o++] = to_base64[sextet[i]];
		for ( ; index<3; ++index )
			dst[o++] = PAD;
	}
	dst[o] = '\0';
/*
fprintf(stderr, "into %d chars\n", o+1);
*/
	return(o);
}

unsigned int base64_decode(const char *src, unsigned int srclen, char *dst, unsigned int dstlen)
{
	char sextet[4], obyte;
	unsigned int i, index=0;
	unsigned int o=0;

	/* No blank lines, please */
	if ( (*src == '\0') || (*src == '\n') )
		return(0);

	/* Now loop, decoding. */
/*
fprintf(stderr, "Decoding %d chars...", srclen);
*/
	for ( i=0; i<srclen; ++i ) {
		if ( src[i] >= 'A' && src[i] <= 'Z' )
			sextet[index++]=(src[i]-'A');
		else if ( src[i] >= 'a' && src[i] <= 'z' )
			sextet[index++]=(src[i]-'a'+26);
		else if ( src[i] >= '0' && src[i] <= '9' )
			sextet[index++]=(src[i]-'0'+52);
		else {
			switch (src[i]) {
				case '+': sextet[index++]=62;
					  	  break;
				case '/': sextet[index++]=63;
					  	  break;
				case '-':
				case PAD: /* End of input.  Flush any remaining data. */
						  switch(index) {
							case 0: break;
							case 1: /* Should never happen */
									break;
							case 2: obyte=(sextet[0]&0x3f);
									obyte <<= 2;
									obyte |= ((sextet[1]&0x30)>>4);
									dst[o++] = obyte;
									break;
							case 3: obyte=(sextet[0]&0x3f);
									obyte <<= 2;
									obyte |= ((sextet[1]&0x30)>>4);
									dst[o++] = obyte;
									obyte=(sextet[1]&0x0f);
									obyte <<= 4;
									obyte |= ((sextet[2]&0x3c)>>2);
									dst[o++] = obyte;
									break;
			  			   }
/*
fprintf(stderr, "into %d chars\n", *olen);
*/
			  			   return(o);
				/* RFC 1521 says ignore junk */
				default:  break;
			}
		}
		if ( index == 4 ) {	/* Output the decoded data */
			obyte=(sextet[0]&0x3f);
			obyte <<= 2;
			obyte |= ((sextet[1]&0x30)>>4);
			dst[o++] = obyte;
			obyte=(sextet[1]&0x0f);
			obyte <<= 4;
			obyte |= ((sextet[2]&0x3c)>>2);
			dst[o++] = obyte;
			obyte=(sextet[2]&0x03);
			obyte <<= 6;
			obyte |= (sextet[3]&0x3f);
			dst[o++] = obyte;
			index=0;
		}
	}
/*
fprintf(stderr, "into %d chars\n", o);
*/
	return(o);
}

