
#include <sys/types.h>
#include <regex.h>

static const char *posix_match(const char *string, const char *pattern, int *matchlen, int flags)
{
	int error;
	regex_t patbuf;
	regmatch_t match;

	/* No string, no pattern, no service. :) */
	if ( ! string || ! pattern ) {
		return((char *)0);
	}

	/* Create the regular expression object */
	if ( regcomp(&patbuf, pattern, flags) < 0 ) {
		return((char *)0);
	}

	/* Check for a match */
	error = regexec(&patbuf, string, 1, &match, 0);
	regfree(&patbuf);
	if ( error || (match.rm_so < 0) ) {
		return((char *)0);
	}
	*matchlen = (match.rm_eo - match.rm_so);
	return(string+match.rm_so);
}

/* Perform case-sensitive string matching */
const char *match(const char *string, const char *pattern, int *matchlen)
{
	return(posix_match(string, pattern, matchlen, 0));
}

/* Perform case-insensitive string matching */
const char *match_nocase(const char *string, const char *pattern, int *matchlen)
{
	return(posix_match(string, pattern, matchlen, REG_ICASE));
}
